import * as ts from "typescript";
import { ApiBaseDefinition, TypeScriptTypeDeclarationDebug } from "../contracts/api-definitions";
import { ApiMetadataDto } from "../contracts/api-metadata-dto";
import { ExtractorOptions } from "../contracts/extractor-options";
import { ReadonlyRegistry } from "../contracts/registry";
export interface ApiItemOptions {
    Program: ts.Program;
    ExtractorOptions: ExtractorOptions;
    ExternalPackages: string[];
    Registry: ReadonlyRegistry<ApiItem>;
    AddItemToRegistry(item: ApiItem<ts.Declaration, any>): string;
}
export declare enum ApiItemStatus {
    Initial = 0,
    Gathered = 1,
    Extracted = 2,
    GatheredAndExtracted = 3,
}
export declare abstract class ApiItem<TDeclaration extends ts.Declaration = ts.Declaration, TExtractDto = ApiBaseDefinition> {
    private declaration;
    private symbol;
    private options;
    constructor(declaration: TDeclaration, symbol: ts.Symbol, options: ApiItemOptions);
    protected TypeChecker: ts.TypeChecker;
    protected ItemStatus: ApiItemStatus;
    protected ExtractedData: TExtractDto;
    GetItemMetadata(): ApiMetadataDto;
    readonly Options: ApiItemOptions;
    readonly Declaration: TDeclaration;
    readonly Symbol: ts.Symbol;
    readonly Status: ApiItemStatus;
    /**
     * It gives precise TypeScript information about item after extracting.
     */
    protected GetTsDebugInfo(): TypeScriptTypeDeclarationDebug | undefined;
    protected abstract OnExtract(): TExtractDto;
    Extract(forceExtraction?: boolean): TExtractDto;
    protected abstract OnGatherData(): void;
    GatherData(forceGathering?: boolean): void;
}
