import * as ts from "typescript";
import { ApiItemOptions } from "./abstractions/api-item";
import { ApiType, ApiTypeKind, ApiBaseType, ApiBasicTypeDto, ApiReferenceBaseType, ApiReferenceTypeDto, ApiUnionOrIntersectionTypeDto, IndexedAccessTypeDto, ArrayTypeDto, ParenthesizedTypeDto, TupleTypeDto, TypeOperatorTypeDto, TypePredicateTypeDto, TypeQueryTypeDto } from "./contracts/api-types";
import { ApiItemLocationDto } from "./contracts/api-item-location-dto";
export declare namespace ApiTypeHelpers {
    /**
     * Resolves ApiType from TypeScript type system.
     * @param options ApiItem otpions
     * @param location ApiItem location for fallback if typeNode is synthesized.
     * @param type Type from TypeScript type system.
     * @param typeNode Type node.
     * @param self For ApiTypeAlias only. Not to have reference to itself.
     */
    function ResolveApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode?: ts.TypeNode, self?: boolean): ApiType;
    /**
     * @internal
     */
    function typeNodeToBaseType(options: ApiItemOptions, apiItemlocation: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeNode): ApiBaseType;
    /**
     * @internal
     */
    interface ResolvedTypeItemReference {
        name?: string;
        referenceId?: string;
    }
    /**
     * @internal
     */
    function resolveTypeItemReference(symbol: ts.Symbol, options: ApiItemOptions): ResolvedTypeItemReference;
    function ResolveApiBasicType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeNode): ApiBasicTypeDto;
    /**
     * Resolving `Definition` reference.
     * @param kind `ApiReferenceBaseType` ApiTypeKind value
     */
    function ReferenceBaseTypeToTypeDto(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeNode, kind?: ApiTypeKind): ApiReferenceBaseType;
    /**
     * Example `Foo`.
     */
    function TypeReferenceNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeReferenceType, self?: boolean): ApiReferenceTypeDto;
    /**
     * Example `string | number` or `string & number`.
     */
    function TypeNodeUnionOrIntersectionToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.UnionOrIntersectionType, typeNode: ts.UnionTypeNode | ts.IntersectionTypeNode): ApiUnionOrIntersectionTypeDto;
    /**
     * Example `string[]`.
     */
    function ArrayTypeNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.ArrayTypeNode): ArrayTypeDto;
    /**
     * Example `[string, number]`.
     */
    function TupleTypeNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TupleTypeNode): TupleTypeDto;
    /**
     * Example `arg is string`.
     */
    function TypePredicateNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypePredicateNode): TypePredicateTypeDto;
    /**
     * Example `keyof Foo`.
     */
    function TypeOperatorNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeOperatorNode): TypeOperatorTypeDto;
    /**
     * Example: `Foo[T]`.
     */
    function IndexedAccessTypeNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.IndexedAccessTypeNode): IndexedAccessTypeDto;
    /**
     * Example: `(string | number)`.
     */
    function ParenthesizedTypeNodeToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.ParenthesizedTypeNode): ParenthesizedTypeDto;
    /**
     * Example: `typeof Foo`.
     */
    function TypeQueryToApiType(options: ApiItemOptions, location: ApiItemLocationDto, type: ts.Type, typeNode: ts.TypeQueryNode): TypeQueryTypeDto;
    type HeritageKinds = ts.SyntaxKind.ImplementsKeyword | ts.SyntaxKind.ExtendsKeyword;
    /**
     * Gets heritage list by kind.
     * @param options Api item options.
     * @param location ApiItem location.
     * @param heritageClauses Array of heritage clauses.
     * @param kind Implements or Extends keyword.
     */
    function GetHeritageList(options: ApiItemOptions, location: ApiItemLocationDto, heritageClauses: ts.NodeArray<ts.HeritageClause>, kind: HeritageKinds): ApiType[];
}
